"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _graphqlTag = _interopRequireDefault(require("graphql-tag"));

var _immutabilityHelper = _interopRequireDefault(require("immutability-helper"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { if (!(Symbol.iterator in Object(arr) || Object.prototype.toString.call(arr) === "[object Arguments]")) { return; } var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var CACHE_KEY_DELIMITER = '%?%?%';
var queryCache = {};

var generateCacheKey = function generateCacheKey(query) {
  var variables = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  return "".concat(query).concat(CACHE_KEY_DELIMITER).concat(JSON.stringify(variables));
};

var parseCacheKey = function parseCacheKey(cacheKey) {
  var _cacheKey$split = cacheKey.split(CACHE_KEY_DELIMITER),
      _cacheKey$split2 = _slicedToArray(_cacheKey$split, 2),
      query = _cacheKey$split2[0],
      variables = _cacheKey$split2[1];

  return {
    query: query,
    variables: JSON.parse(variables)
  };
};

var _default = {
  setActive: function setActive(query, variables) {
    var data = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    var cacheKey = generateCacheKey(query, variables);

    if (queryCache[cacheKey]) {
      queryCache[cacheKey].activeCount += 1;
    } else {
      queryCache[cacheKey] = {
        isDirty: false,
        activeCount: 1,
        variables: variables,
        data: data,
        paginationCursor: null
      };
    }
  },
  setInactive: function setInactive(query, variables) {
    var cacheKey = generateCacheKey(query, variables);
    queryCache[cacheKey].activeCount -= 1;
  },
  isActive: function isActive(query, variables) {
    var cacheKey = generateCacheKey(query, variables);
    return !!queryCache[cacheKey] && queryCache[cacheKey].activeCount > 0;
  },
  isDirty: function isDirty(query, variables) {
    var cacheKey = generateCacheKey(query, variables);
    return !!queryCache[cacheKey] && queryCache[cacheKey].isDirty;
  },
  setClean: function setClean(query, variables) {
    var cacheKey = generateCacheKey(query, variables);
    queryCache[cacheKey].isDirty = false;
  },
  setDirty: function setDirty(items, apolloClient) {
    Object.entries(queryCache).forEach(function (_ref) {
      var _ref2 = _slicedToArray(_ref, 2),
          cacheKey = _ref2[0],
          _ref2$ = _ref2[1],
          data = _ref2$.data,
          activeCount = _ref2$.activeCount;

      if (data.dirtyWhen && data.dirtyWhen.some(function (item) {
        return items.includes(item);
      })) {
        if (activeCount > 0) {
          var _parseCacheKey = parseCacheKey(cacheKey),
              query = _parseCacheKey.query,
              variables = _parseCacheKey.variables;

          apolloClient.query({
            query: (0, _graphqlTag["default"])(query),
            variables: variables,
            fetchPolicy: 'network-only',
            context: data.context
          });
          queryCache[cacheKey].paginationCursor = null;
        } else {
          queryCache[cacheKey].isDirty = true;
        }
      }
    });
  },
  getPaginationCursor: function getPaginationCursor(query, variables) {
    var cacheKey = generateCacheKey(query, variables);
    return queryCache[cacheKey] ? queryCache[cacheKey].paginationCursor : null;
  },
  setPaginationCursor: function setPaginationCursor(query, variables, cursor) {
    var cacheKey = generateCacheKey(query, variables);
    queryCache[cacheKey].paginationCursor = cursor;
  },
  handleEvents: function handleEvents(mutationName, item, mutationVariables, apolloClient) {
    if (window.__SOFTMIGRATION__triggerQueryObserver) {
      window.__SOFTMIGRATION__triggerQueryObserver(mutationName, item);
    }

    Object.entries(queryCache).forEach(function (_ref3) {
      var _ref4 = _slicedToArray(_ref3, 2),
          cacheKey = _ref4[0],
          _ref4$ = _ref4[1],
          data = _ref4$.data,
          activeCount = _ref4$.activeCount;

      if (data.events) {
        data.events.forEach(function (event) {
          if (event.on === mutationName) {
            if (event.cacheUpdatePath) {
              var _parseCacheKey2 = parseCacheKey(cacheKey),
                  query = _parseCacheKey2.query,
                  variables = _parseCacheKey2.variables;

              var queryAst = (0, _graphqlTag["default"])(query);
              var cacheData = apolloClient.readQuery({
                query: queryAst,
                variables: variables
              });
              apolloClient.writeQuery({
                query: queryAst,
                data: (0, _immutabilityHelper["default"])(cacheData, event.cacheUpdatePath({
                  item: item,
                  cacheData: cacheData,
                  queryVariables: variables,
                  mutationVariables: mutationVariables
                })),
                variables: variables
              });
            }

            if (event.refetch) {
              if (activeCount > 0) {
                var _parseCacheKey3 = parseCacheKey(cacheKey),
                    _query = _parseCacheKey3.query,
                    _variables = _parseCacheKey3.variables;

                apolloClient.query({
                  query: (0, _graphqlTag["default"])(_query),
                  variables: _variables,
                  fetchPolicy: 'network-only',
                  context: data.context
                });
                queryCache[cacheKey].paginationCursor = null;
              } else {
                queryCache[cacheKey].isDirty = true;
              }
            }
          }
        });
      }
    });
  },
  getAllVariables: function getAllVariables(query) {
    return Object.entries(queryCache).reduce(function (result, _ref5) {
      var _ref6 = _slicedToArray(_ref5, 2),
          cacheKey = _ref6[0],
          data = _ref6[1];

      return cacheKey.startsWith(query) ? [].concat(_toConsumableArray(result), [data.variables]) : result;
    }, []);
  }
};
exports["default"] = _default;